@@ -23,20 +23,36 @@ from __future__ import with_statement
 import contextlib
 import os
 import shutil
+import sys
 
-import dbus
-import dbus.service
-
-from dbus.mainloop.glib import DBusGMainLoop
+from functools import wraps
 
 from twisted.internet import defer
 from twisted.python import failure
-from twisted.trial.unittest import TestCase
+from twisted.trial.unittest import TestCase, SkipTest
 
 # DBusRunner for DBusTestCase using tests
 from ubuntuone.devtools.services.dbus import DBusRunner
 
 
+# pylint: disable=F0401,C0103
+try:
+    import dbus
+except ImportError:
+    dbus = None
+
+try:
+    import dbus.service as service
+except ImportError:
+    service = None
+
+try:
+    from dbus.mainloop.glib import DBusGMainLoop
+except ImportError:
+    DBusGMainLoop = None
+
+
+# pylint: enable=F0401,C0103
 @contextlib.contextmanager
 def environ(env_var, new_value):
     """context manager to replace/add an environ value"""
@@ -49,6 +65,70 @@ def environ(env_var, new_value):
         os.environ[env_var] = old_value
 
 
+def _id(obj):
+    """Return the obj calling the funct."""
+    return obj
+
+
+# pylint: disable=C0103
+def skipTest(reason):
+    """Unconditionally skip a test."""
+
+    def decorator(test_item):
+        """Decorate the test so that it is skipped."""
+        if not (isinstance(test_item, type) and\
+            issubclass(test_item, TestCase)):
+
+            @wraps(test_item)
+            def skip_wrapper(*args, **kwargs):
+                """Skip a test method raising an exception."""
+                raise SkipTest(reason)
+            test_item = skip_wrapper
+
+        # tell twisted.trial.unittest to skip the test, pylint will complain
+        # since it thinks we are redefining a name out of the scope
+        # pylint: disable=W0621,W0612
+        test_item.skip = reason
+        # pylint: enable=W0621,W0612
+        # because the item was skipped, we will make sure that no
+        # services are started for it
+        if hasattr(test_item, "required_services"):
+            # pylint: disable=W0612
+            test_item.required_services = lambda *args, **kwargs: []
+            # pylint: enable=W0612
+        return test_item
+    return decorator
+
+
+def skipIf(condition, reason):
+    """Skip a test if the condition is true."""
+    if condition:
+        return skipTest(reason)
+    return _id
+
+
+def skipIfOS(current_os, reason):
+    """Skip test for a particular os or lists of them."""
+    if os:
+        if sys.platform in current_os or sys.platform == current_os:
+            return skipTest(reason)
+        return _id
+    return _id
+
+
+def skipIfNotOS(current_os, reason):
+    """Skip test we are not in a particular os."""
+    if os:
+        if sys.platform not in current_os or\
+            sys.platform != current_os:
+            return skipTest(reason)
+        return _id
+    return _id
+
+
+# pylint: enable=C0103
+
+
 class FakeDBusInterface(object):
     """A fake DBusInterface..."""
 
@@ -118,6 +198,8 @@ class BaseTestCase(TestCase):
         os.makedirs(path)
 
 
+@skipIf(dbus is None or service is None or DBusGMainLoop is None,
+    "The test requires dbus.")
 class DBusTestCase(BaseTestCase):
     """Test the DBus event handling."""
 
@@ -131,12 +213,14 @@ class DBusTestCase(BaseTestCase):
         """Setup the infrastructure fo the test (dbus service)."""
         # Class 'BaseTestCase' has no 'setUp' member
         # pylint: disable=E1101
+        # dbus modules will be imported by the decorator
+        # pylint: disable=E0602
         BaseTestCase.setUp(self)
         self.loop = DBusGMainLoop(set_as_default=True)
         self.bus = dbus.bus.BusConnection(mainloop=self.loop)
         # monkeypatch busName.__del__ to avoid errors on gc
         # we take care of releasing the name in shutdown
-        dbus.service.BusName.__del__ = lambda _: None
+        service.BusName.__del__ = lambda _: None
         self.bus.set_exit_on_disconnect(False)
         self.signal_receivers = set()
 
@@ -161,6 +245,8 @@ class DBusTestCase(BaseTestCase):
 
     def cleanup_signal_receivers(self, signal_receivers):
         """Cleanup self.signal_receivers and returns a deferred."""
+        # dbus modules will be imported by the decorator
+        # pylint: disable=E0602
         deferreds = []
         for match in signal_receivers:
             d = defer.Deferred()
